/* Auto generated file: with make_docs.py .  Docs go in docs/reST/ref/ . */
#define DOC_JOYSTICK "Pygame module for interacting with joysticks, gamepads, and trackballs."
#define DOC_JOYSTICK_INIT "init() -> None\nInitialize the joystick module."
#define DOC_JOYSTICK_QUIT "quit() -> None\nUninitialize the joystick module."
#define DOC_JOYSTICK_GETINIT "get_init() -> bool\nReturns True if the joystick module is initialized."
#define DOC_JOYSTICK_GETCOUNT "get_count() -> count\nReturns the number of joysticks."
#define DOC_JOYSTICK_JOYSTICK "Joystick(id, /) -> Joystick\nCreate a new Joystick object."
#define DOC_JOYSTICK_JOYSTICK_INIT "init() -> None\ninitialize the Joystick"
#define DOC_JOYSTICK_JOYSTICK_QUIT "quit() -> None\nuninitialize the Joystick"
#define DOC_JOYSTICK_JOYSTICK_GETINIT "get_init() -> bool\ncheck if the Joystick is initialized"
#define DOC_JOYSTICK_JOYSTICK_GETID "get_id() -> int\nget the device index (deprecated)"
#define DOC_JOYSTICK_JOYSTICK_GETINSTANCEID "get_instance_id() -> int\nget the joystick instance id"
#define DOC_JOYSTICK_JOYSTICK_GETGUID "get_guid() -> str\nget the joystick GUID"
#define DOC_JOYSTICK_JOYSTICK_GETPOWERLEVEL "get_power_level() -> str\nget the approximate power status of the device"
#define DOC_JOYSTICK_JOYSTICK_GETNAME "get_name() -> string\nget the Joystick system name"
#define DOC_JOYSTICK_JOYSTICK_GETNUMAXES "get_numaxes() -> int\nget the number of axes on a Joystick"
#define DOC_JOYSTICK_JOYSTICK_GETAXIS "get_axis(axis_number, /) -> float\nget the current position of an axis"
#define DOC_JOYSTICK_JOYSTICK_GETNUMBALLS "get_numballs() -> int\nget the number of trackballs on a Joystick"
#define DOC_JOYSTICK_JOYSTICK_GETBALL "get_ball(ball_number, /) -> x, y\nget the relative position of a trackball"
#define DOC_JOYSTICK_JOYSTICK_GETNUMBUTTONS "get_numbuttons() -> int\nget the number of buttons on a Joystick"
#define DOC_JOYSTICK_JOYSTICK_GETBUTTON "get_button(button, /) -> bool\nget the current button state"
#define DOC_JOYSTICK_JOYSTICK_GETNUMHATS "get_numhats() -> int\nget the number of hat controls on a Joystick"
#define DOC_JOYSTICK_JOYSTICK_GETHAT "get_hat(hat_number, /) -> x, y\nget the position of a joystick hat"
#define DOC_JOYSTICK_JOYSTICK_RUMBLE "rumble(low_frequency, high_frequency, duration) -> bool\nStart a rumbling effect"
#define DOC_JOYSTICK_JOYSTICK_STOPRUMBLE "stop_rumble() -> None\nStop any rumble effect playing"
