/* Auto generated file: with make_docs.py .  Docs go in docs/reST/ref/ . */
#define DOC_PIXELCOPY "pygame module for general pixel array copying"
#define DOC_PIXELCOPY_SURFACETOARRAY "surface_to_array(array, surface, kind='P', opaque=255, clear=0) -> None\ncopy surface pixels to an array object"
#define DOC_PIXELCOPY_ARRAYTOSURFACE "array_to_surface(surface, array, /) -> None\ncopy an array object to a surface"
#define DOC_PIXELCOPY_MAPARRAY "map_array(array, array, surface, /) -> None\ncopy an array to another array, using surface format"
#define DOC_PIXELCOPY_MAKESURFACE "pygame.pixelcopy.make_surface(array, /) -> Surface\nCopy an array to a new surface"
