/* Auto generated file: with make_docs.py .  Docs go in docs/reST/ref/ . */
#define DOC_BUFFERPROXY "BufferProxy(<parent>) -> BufferProxy\npygame object to export a surface buffer through an array protocol"
#define DOC_BUFFERPROXY_PARENT "parent -> Surface\nparent -> <parent>\nReturn wrapped exporting object."
#define DOC_BUFFERPROXY_LENGTH "length -> int\nThe size, in bytes, of the exported buffer."
#define DOC_BUFFERPROXY_RAW "raw -> bytes\nA copy of the exported buffer as a single block of bytes."
#define DOC_BUFFERPROXY_WRITE "write(buffer, offset=0)\nWrite raw bytes to object buffer."
