/* Auto generated file: with make_docs.py .  Docs go in docs/reST/ref/ . */
#define DOC_TIME "Pygame module for monitoring time."
#define DOC_TIME_GETTICKS "get_ticks() -> int\nGet the time in milliseconds."
#define DOC_TIME_WAIT "wait(milliseconds, /) -> int\nPause the program for an amount of time."
#define DOC_TIME_DELAY "delay(milliseconds, /) -> int\nPause the program for an amount of time."
#define DOC_TIME_SETTIMER "set_timer(event, millis, loops=0) -> None\nRepeatedly create an event on the event queue."
#define DOC_TIME_CLOCK "Clock() -> Clock\nCreate an object to help track time."
#define DOC_TIME_CLOCK_TICK "tick(framerate=0, /) -> int\nUpdate the clock."
#define DOC_TIME_CLOCK_TICKBUSYLOOP "tick_busy_loop(framerate=0, /) -> int\nUpdate the clock."
#define DOC_TIME_CLOCK_GETTIME "get_time() -> int\nTime used in the previous tick."
#define DOC_TIME_CLOCK_GETRAWTIME "get_rawtime() -> int\nActual time used in the previous tick."
#define DOC_TIME_CLOCK_GETFPS "get_fps() -> float\nCompute the clock framerate."
