/* Auto generated file: with make_docs.py .  Docs go in docs/reST/ref/ . */
#define DOC_EXAMPLES "module of example programs"
#define DOC_EXAMPLES_ALIENS_MAIN "aliens.main() -> None\nplay the full aliens example"
#define DOC_EXAMPLES_STARS_MAIN "stars.main() -> None\nrun a simple starfield example"
#define DOC_EXAMPLES_CHIMP_MAIN "chimp.main() -> None\nhit the moving chimp"
#define DOC_EXAMPLES_MOVEIT_MAIN "moveit.main() -> None\ndisplay animated objects on the screen"
#define DOC_EXAMPLES_FONTY_MAIN "fonty.main() -> None\nrun a font rendering example"
#define DOC_EXAMPLES_FREETYPEMISC_MAIN "freetype_misc.main() -> None\nrun a FreeType rendering example"
#define DOC_EXAMPLES_VGRADE_MAIN "vgrade.main() -> None\ndisplay a vertical gradient"
#define DOC_EXAMPLES_EVENTLIST_MAIN "eventlist.main() -> None\ndisplay pygame events"
#define DOC_EXAMPLES_ARRAYDEMO_MAIN "arraydemo.main(arraytype=None) -> None\nshow various surfarray effects"
#define DOC_EXAMPLES_SOUND_MAIN "sound.main(file_path=None) -> None\nload and play a sound"
#define DOC_EXAMPLES_SOUNDARRAYDEMOS_MAIN "sound_array_demos.main(arraytype=None) -> None\nplay various sndarray effects"
#define DOC_EXAMPLES_LIQUID_MAIN "liquid.main() -> None\ndisplay an animated liquid effect"
#define DOC_EXAMPLES_GLCUBE_MAIN "glcube.main() -> None\ndisplay an animated 3D cube using OpenGL"
#define DOC_EXAMPLES_SCRAPCLIPBOARD_MAIN "scrap_clipboard.main() -> None\naccess the clipboard"
#define DOC_EXAMPLES_MASK_MAIN "mask.main(*args) -> None\ndisplay multiple images bounce off each other using collision detection"
#define DOC_EXAMPLES_TESTSPRITE_MAIN "testsprite.main(update_rects = True, use_static = False, use_FastRenderGroup = False, screen_dims = [640, 480], use_alpha = False, flags = 0) -> None\nshow lots of sprites moving around"
#define DOC_EXAMPLES_HEADLESSNOWINDOWSNEEDED_MAIN "headless_no_windows_needed.main(fin, fout, w, h) -> None\nwrite an image file that is smoothscaled copy of an input file"
#define DOC_EXAMPLES_JOYSTICK_MAIN "joystick.main() -> None\ndemonstrate joystick functionality"
#define DOC_EXAMPLES_BLENDFILL_MAIN "blend_fill.main() -> None\ndemonstrate the various surface.fill method blend options"
#define DOC_EXAMPLES_BLITBLENDS_MAIN "blit_blends.main() -> None\nuses alternative additive fill to that of surface.fill"
#define DOC_EXAMPLES_CURSORS_MAIN "cursors.main() -> None\ndisplay two different custom cursors"
#define DOC_EXAMPLES_PIXELARRAY_MAIN "pixelarray.main() -> None\ndisplay various pixelarray generated effects"
#define DOC_EXAMPLES_SCALETEST_MAIN "scaletest.main(imagefile, convert_alpha=False, run_speed_test=True) -> None\ninteractively scale an image using smoothscale"
#define DOC_EXAMPLES_MIDI_MAIN "midi.main(mode='output', device_id=None) -> None\nrun a midi example"
#define DOC_EXAMPLES_SCROLL_MAIN "scroll.main(image_file=None) -> None\nrun a Surface.scroll example that shows a magnified image"
#define DOC_EXAMPLES_CAMERA_MAIN "camera.main() -> None\ndisplay video captured live from an attached camera"
#define DOC_EXAMPLES_PLAYMUS_MAIN "playmus.main(file_path) -> None\nplay an audio file"
