/* Auto generated file: with make_docs.py .  Docs go in docs/reST/ref/ . */
#define DOC_SNDARRAY "pygame module for accessing sound sample data"
#define DOC_SNDARRAY_ARRAY "array(Sound) -> array\ncopy Sound samples into an array"
#define DOC_SNDARRAY_SAMPLES "samples(Sound) -> array\nreference Sound samples into an array"
#define DOC_SNDARRAY_MAKESOUND "make_sound(array) -> Sound\nconvert an array into a Sound object"
#define DOC_SNDARRAY_USEARRAYTYPE "use_arraytype (arraytype) -> None\nSets the array system to be used for sound arrays"
#define DOC_SNDARRAY_GETARRAYTYPE "get_arraytype () -> str\nGets the currently active array type."
#define DOC_SNDARRAY_GETARRAYTYPES "get_arraytypes () -> tuple\nGets the array system types currently supported."
