from typing_extensions import deprecated # added in 3.13

import numpy

from pygame.mixer import Sound

def array(sound: Sound) -> numpy.ndarray: ...
def samples(sound: Sound) -> numpy.ndarray: ...
def make_sound(array: numpy.ndarray) -> Sound: ...
@deprecated("Only numpy is supported")
def use_arraytype(arraytype: str) -> Sound: ...
@deprecated("Only numpy is supported")
def get_arraytype() -> str: ...
@deprecated("Only numpy is supported")
def get_arraytypes() -> tuple[str]: ...
