/* Auto generated file: with make_docs.py .  Docs go in docs/reST/ref/ . */
#define DOC_CURSORS "Pygame module for cursor resources."
#define DOC_CURSORS_COMPILE "compile(strings, black='X', white='.', xor='o') -> tuple\nCreate binary cursor data from simple strings."
#define DOC_CURSORS_LOADXBM "load_xbm(curs, mask) -> tuple\nLoad cursor data from an XBM file."
#define DOC_CURSORS_CURSOR "Cursor(constant=...) -> Cursor\nCursor(cursor) -> Cursor\nCursor(size, hotspot, xormasks, andmasks) -> Cursor\nCursor(hotspot, surface) -> Cursor\nPygame object representing a cursor."
#define DOC_CURSORS_CURSOR_COPY "copy() -> Cursor\nCopy the current cursor."
#define DOC_CURSORS_CURSOR_TYPE "type -> Literal['system', 'color', 'bitmap']\nGet the cursor type."
#define DOC_CURSORS_CURSOR_DATA "data -> ...\nGet the cursor data."
