/* Auto generated file: with make_docs.py .  Docs go in docs/reST/ref/ . */
#define DOC_WINDOW "Window(title='pygame window', size=(640, 480), position=None, fullscreen=False, fullscreen_desktop=False, **kwargs) -> Window\npygame object that represents a window"
#define DOC_WINDOW_GRABMOUSE "grab_mouse -> bool\nGet or set the window's mouse grab mode"
#define DOC_WINDOW_GRABKEYBOARD "grab_keyboard -> bool\nGet or set the window's keyboard grab mode"
#define DOC_WINDOW_MOUSEGRABBED "mouse_grabbed -> bool\nGet if the mouse cursor is confined to the window (**read-only**)"
#define DOC_WINDOW_KEYBOARDGRABBED "keyboard_grabbed -> bool\nGet if the keyboard shortcuts are captured by the window (**read-only**)"
#define DOC_WINDOW_FOCUSED "focused -> bool\nGet if the window is focused (**read-only**)"
#define DOC_WINDOW_TITLE "title -> str\nGet or set the window title"
#define DOC_WINDOW_RESIZABLE "resizable -> bool\nGet or set whether the window is resizable"
#define DOC_WINDOW_BORDERLESS "borderless -> bool\nGet or set whether the window is borderless"
#define DOC_WINDOW_ALWAYSONTOP "always_on_top -> bool\nGet or set whether the window is always on top"
#define DOC_WINDOW_ID "id -> int\nGet the unique window ID (**read-only**)"
#define DOC_WINDOW_MOUSERECT "mouse_rect -> Rect|None\nGet or set the mouse confinement rectangle of the window"
#define DOC_WINDOW_SIZE "size -> (int, int)\nGet or set the window size in pixels"
#define DOC_WINDOW_MINIMUMSIZE "minimum_size -> (int, int)\nGet or set the minimum size of the window's client area"
#define DOC_WINDOW_MAXIMUMSIZE "maximum_size -> (int, int)\nGet or set the maximum size of the window's client area"
#define DOC_WINDOW_POSITION "position -> (int, int) or WINDOWPOS_CENTERED or WINDOWPOS_UNDEFINED\nGet or set the window position in screen coordinates"
#define DOC_WINDOW_OPACITY "opacity -> float\nGet or set the window opacity, between 0.0 (fully transparent) and 1.0 (fully opaque)"
#define DOC_WINDOW_OPENGL "opengl -> bool\nGet if the window supports OpenGL"
#define DOC_WINDOW_UTILITY "utility -> bool\nGet if the window is an utility window (**read-only**)"
#define DOC_WINDOW_FROMDISPLAYMODULE "from_display_module() -> Window\nCreate a Window object using window data from display module"
#define DOC_WINDOW_GETSURFACE "get_surface() -> Surface\nGet the window surface"
#define DOC_WINDOW_FLIP "flip() -> None\nUpdate the display surface to the window."
#define DOC_WINDOW_SETWINDOWED "set_windowed() -> None\nEnable windowed mode (exit fullscreen)"
#define DOC_WINDOW_SETFULLSCREEN "set_fullscreen(desktop=False) -> None\nEnter fullscreen"
#define DOC_WINDOW_DESTROY "destroy() -> None\nDestroy the window"
#define DOC_WINDOW_HIDE "hide() -> None\nHide the window"
#define DOC_WINDOW_SHOW "show() -> None\nShow the window"
#define DOC_WINDOW_FOCUS "focus(input_only=False) -> None\nSet the window to be focused"
#define DOC_WINDOW_RESTORE "restore() -> None\nRestore the size and position of a minimized or maximized window"
#define DOC_WINDOW_MAXIMIZE "maximize() -> None\nMaximize the window"
#define DOC_WINDOW_MINIMIZE "maximize() -> None\nMinimize the window"
#define DOC_WINDOW_SETICON "set_icon(surface, /) -> None\nSet the window icon"
#define DOC_WINDOW_SETMODALFOR "set_modal_for(parent, /) -> None\nSet the window as a modal for a parent window"
#define DOC_WINDOW_FLASH "flash(operation, /) -> None\nFlash a window to demand attention from the user"
