/* Auto generated file: with make_docs.py .  Docs go in docs/reST/ref/ . */
#define DOC_CAMERA "pygame module for camera use"
#define DOC_CAMERA_INIT "init(backend = None) -> None\nModule init"
#define DOC_CAMERA_GETBACKENDS "get_backends() -> [str]\nGet the backends supported on this system"
#define DOC_CAMERA_COLORSPACE "colorspace(surface, format, dest_surface = None, /) -> Surface\nSurface colorspace conversion"
#define DOC_CAMERA_LISTCAMERAS "list_cameras() -> [cameras]\nreturns a list of available cameras"
#define DOC_CAMERA_CAMERA "Camera(device, (width, height), format) -> Camera\nload a camera"
#define DOC_CAMERA_CAMERA_START "start() -> None\nopens, initializes, and starts capturing"
#define DOC_CAMERA_CAMERA_STOP "stop() -> None\nstops, uninitializes, and closes the camera"
#define DOC_CAMERA_CAMERA_GETCONTROLS "get_controls() -> (hflip = bool, vflip = bool, brightness)\ngets current values of user controls"
#define DOC_CAMERA_CAMERA_SETCONTROLS "set_controls(hflip = bool, vflip = bool, brightness) -> (hflip = bool, vflip = bool, brightness)\nchanges camera settings if supported by the camera"
#define DOC_CAMERA_CAMERA_GETSIZE "get_size() -> (width, height)\nreturns the dimensions of the images being recorded"
#define DOC_CAMERA_CAMERA_QUERYIMAGE "query_image() -> bool\nchecks if a frame is ready"
#define DOC_CAMERA_CAMERA_GETIMAGE "get_image(Surface = None, /) -> Surface\ncaptures an image as a Surface"
#define DOC_CAMERA_CAMERA_GETRAW "get_raw() -> bytes\nreturns an unmodified image as bytes"
