from typing import TypedDict, Optional

# dict at runtime, this only exist for benefit of the typechecker
class _FingerDict(TypedDict):
    id: int
    x: float
    y: float
    pressure: float

def get_num_devices() -> int: ...
def get_device(index: int, /) -> int: ...
def get_num_fingers(device_id: int, /) -> int: ...
def get_finger(touchid: int, index: int) -> Optional[_FingerDict]: ...
