/* Auto generated file: with make_docs.py .  Docs go in docs/reST/ref/ . */
#define DOC_SURFARRAY "pygame module for accessing surface pixel data using array interfaces"
#define DOC_SURFARRAY_ARRAY2D "array2d(Surface) -> array\nCopy pixels into a 2d array"
#define DOC_SURFARRAY_PIXELS2D "pixels2d(Surface) -> array\nReference pixels into a 2d array"
#define DOC_SURFARRAY_ARRAY3D "array3d(Surface) -> array\nCopy pixels into a 3d array"
#define DOC_SURFARRAY_PIXELS3D "pixels3d(Surface) -> array\nReference pixels into a 3d array"
#define DOC_SURFARRAY_ARRAYALPHA "array_alpha(Surface) -> array\nCopy pixel alphas into a 2d array"
#define DOC_SURFARRAY_PIXELSALPHA "pixels_alpha(Surface) -> array\nReference pixel alphas into a 2d array"
#define DOC_SURFARRAY_ARRAYRED "array_red(Surface) -> array\nCopy red pixels into a 2d array"
#define DOC_SURFARRAY_PIXELSRED "pixels_red (Surface) -> array\nReference pixel red into a 2d array."
#define DOC_SURFARRAY_ARRAYGREEN "array_green(Surface) -> array\nCopy green pixels into a 2d array"
#define DOC_SURFARRAY_PIXELSGREEN "pixels_green (Surface) -> array\nReference pixel green into a 2d array."
#define DOC_SURFARRAY_ARRAYBLUE "array_blue(Surface) -> array\nCopy blue pixels into a 2d array"
#define DOC_SURFARRAY_PIXELSBLUE "pixels_blue (Surface) -> array\nReference pixel blue into a 2d array."
#define DOC_SURFARRAY_ARRAYCOLORKEY "array_colorkey(Surface) -> array\nCopy the colorkey values into a 2d array"
#define DOC_SURFARRAY_MAKESURFACE "make_surface(array) -> Surface\nCopy an array to a new surface"
#define DOC_SURFARRAY_BLITARRAY "blit_array(Surface, array) -> None\nBlit directly from a array values"
#define DOC_SURFARRAY_MAPARRAY "map_array(Surface, array3d) -> array2d\nMap a 3d array into a 2d array"
#define DOC_SURFARRAY_USEARRAYTYPE "use_arraytype (arraytype) -> None\nSets the array system to be used for surface arrays"
#define DOC_SURFARRAY_GETARRAYTYPE "get_arraytype () -> str\nGets the currently active array type."
#define DOC_SURFARRAY_GETARRAYTYPES "get_arraytypes () -> tuple\nGets the array system types currently supported."
