/* Auto generated file: with make_docs.py .  Docs go in docs/reST/ref/ . */
#define DOC_SCRAP "pygame module for clipboard support."
#define DOC_SCRAP_PUTTEXT "put_text(text, /) -> None\nPlaces text into the clipboard."
#define DOC_SCRAP_GETTEXT "get_text() -> str\nGets text from the clipboard."
#define DOC_SCRAP_HASTEXT "has_text() -> bool\nChecks if text is in the clipboard."
#define DOC_SCRAP_INIT "init() -> None\nInitializes the scrap module."
#define DOC_SCRAP_GETINIT "get_init() -> bool\nReturns True if the scrap module is currently initialized."
#define DOC_SCRAP_GET "get(type, /) -> bytes | None\nGets the data for the specified type from the clipboard."
#define DOC_SCRAP_GETTYPES "get_types() -> list\nGets a list of the available clipboard types."
#define DOC_SCRAP_PUT "put(type, data, /) -> None\nPlaces data into the clipboard."
#define DOC_SCRAP_CONTAINS "contains(type, /) -> bool\nChecks whether data for a given type is available in the clipboard."
#define DOC_SCRAP_LOST "lost() -> bool\nIndicates if the clipboard ownership has been lost by the pygame application."
#define DOC_SCRAP_SETMODE "set_mode(mode, /) -> None\nSets the clipboard access mode."
